@@ -2,12 +2,12 @@
 BY <- function(x, ...) UseMethod("BY")
 
 
-BY.default <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reorder = TRUE,
+BY.default <- function(x, g, FUN, ..., use.g.names = TRUE, sort = .op[["sort"]], reorder = TRUE,
                        expand.wide = FALSE, parallel = FALSE, mc.cores = 1L,
                        return = c("same", "vector", "list")) {
 
   # If matrix, dispatch to matrix method
-  if(is.matrix(x) && !inherits(x, "matrix")) return(UseMethod("BY", unclass(x)))
+  # if(is.matrix(x) && !inherits(x, "matrix")) return(UseMethod("BY", unclass(x)))
   if(!is.function(FUN)) FUN <- match.fun(FUN)
 
   aplyfun <- if(parallel) function(...) mclapply(..., mc.cores = mc.cores) else lapply
@@ -42,7 +42,7 @@ BY.default <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reorder
 
   } else { # Function does not assign names... or not using group names...
 
-    res <- unlist(res, FALSE, FALSE)
+    res <- funlist(res)
 
     if(length(res) == g[[1L]]) {
 
@@ -66,15 +66,15 @@ BY.default <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reorder
 #   sx <- gsplit(x, g)
 #   if(length(sx) > 100000L && length(r1 <- match.fun(FUN)(sx[[1L]], ...)) == 1L)
 #     return(copyMostAttributes(vapply(sx, FUN, r1, ..., USE.NAMES = FALSE), x))
-#   copyMostAttributes(unlist(lapply(sx, FUN, ...), FALSE, FALSE), x)
+#   copyMostAttributes(funlist(lapply(sx, FUN, ...)), x)
 # }
 
-copysplaplfun <- function(x, g, FUN, ...) copyMostAttributes(unlist(lapply(gsplit(x, g), FUN, ...), FALSE, FALSE), x)
-copysplmaplfun <- function(x, g, FUN, asl, mord) copyMostAttributes(unlist(.mapply(FUN, c(list(gsplit(x, g)), asl), mord), FALSE, FALSE), x)
-splaplfun <- function(x, g, FUN, ...) unlist(lapply(gsplit(x, g), FUN, ...), FALSE, FALSE)
-splmaplfun <- function(x, g, FUN, asl, mord) unlist(.mapply(FUN, c(list(gsplit(x, g)), asl), mord), FALSE, FALSE)
+copysplaplfun <- function(x, g, FUN, ...) copyMostAttributes(funlist(lapply(gsplit(x, g), FUN, ...)), x)
+copysplmaplfun <- function(x, g, FUN, asl, mord) copyMostAttributes(funlist(.mapply(FUN, c(list(gsplit(x, g)), asl), mord)), x)
+splaplfun <- function(x, g, FUN, ...) funlist(lapply(gsplit(x, g), FUN, ...))
+splmaplfun <- function(x, g, FUN, asl, mord) funlist(.mapply(FUN, c(list(gsplit(x, g)), asl), mord))
 
-BY.data.frame <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reorder = TRUE,
+BY.data.frame <- function(x, g, FUN, ..., use.g.names = TRUE, sort = .op[["sort"]], reorder = TRUE,
                           expand.wide = FALSE, parallel = FALSE, mc.cores = 1L,
                           return = c("same", "matrix", "data.frame", "list")) {
 
@@ -118,11 +118,11 @@ BY.data.frame <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reord
         isDTl <- inherits(x, "data.table")
         ax[["names"]] <- names(res)
         ax[["row.names"]] <- if(use.g.names && !isDTl && length(gn <- GRPnames(g))) gn else
-          .set_row_names(length(res[[1L]]))
+          .set_row_names(.Call(C_fnrow, res))
       } else {
         isDTl <- FALSE
         ax <- list(names = names(res),
-                   row.names = if(use.g.names && length(gn <- GRPnames(g))) gn else .set_row_names(length(res[[1L]])),
+                   row.names = if(use.g.names && length(gn <- GRPnames(g))) gn else .set_row_names(.Call(C_fnrow, res)),
                    class = "data.frame")
       }
       return(condalcSA(res, ax, isDTl))
@@ -163,11 +163,11 @@ BY.data.frame <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reord
         reorder <- FALSE
       }
     } else { # function doesn't assign names, different options.
-      res1 <- unlist(res1, FALSE, FALSE)
+      res1 <- funlist(res1)
       if(length(res1) == g[[1L]]) rn <- GRPnames(g)
       else if(matl) {
         rn <- if(length(res1) == n && is.character(rownam) && rownam[1L] != "1" && (reorder || isTRUE(g$ordered[2L]))) rownam else NULL
-      } else {  # Important to check lenth(rn) below (simply keeps ax[["row.names"]])
+      } else {  # Important to check length(rn) below (simply keeps ax[["row.names"]])
         rn <- if(length(res1) != n || !(reorder || isTRUE(g$ordered[2L]))) .set_row_names(length(res1)) else NULL
       }
     }
@@ -188,7 +188,7 @@ BY.data.frame <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reord
       rn <- if(nrow(res) == n && is.character(rownam) && rownam[1L] != "1" && (reorder || isTRUE(g$ordered[2L]))) rownam else NULL
     } else {
       res <- if(multi) aplyfun(x, copysplmaplfun, g, FUN, asl, mord) else aplyfun(x, copysplaplfun, g, FUN, ...) # isDTL ? -> Not needed as data.tables cannot have character row-names anyway.
-      rn <- if(length(res[[1L]]) != n || !(reorder || isTRUE(g$ordered[2L]))) .set_row_names(length(res[[1L]])) else NULL
+      rn <- if(.Call(C_fnrow, res) != n || !(reorder || isTRUE(g$ordered[2L]))) .set_row_names(.Call(C_fnrow, res)) else NULL
     }
   }
 
@@ -209,7 +209,7 @@ BY.data.frame <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reord
 
 BY.list <- function(x, ...) BY.data.frame(x, ...)
 
-BY.matrix <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reorder = TRUE,
+BY.matrix <- function(x, g, FUN, ..., use.g.names = TRUE, sort = .op[["sort"]], reorder = TRUE,
                       expand.wide = FALSE, parallel = FALSE, mc.cores = 1L,
                       return = c("same", "matrix", "data.frame", "list")) {
 
@@ -249,7 +249,7 @@ BY.matrix <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reorder =
         function(y) .Call(Cpp_mctl, do.call(rbind, lapply(gsplit(y, g), FUN, ...)), TRUE, 0L)
       res <- unlist(aplyfun(.Call(Cpp_mctl, x, TRUE, 0L), splitfun), recursive = FALSE, use.names = TRUE)
       ax <- list(names = names(res),
-                 row.names = if(use.g.names && length(gn <- GRPnames(g))) gn else .set_row_names(length(res[[1L]])),
+                 row.names = if(use.g.names && length(gn <- GRPnames(g))) gn else .set_row_names(.Call(C_fnrow, res)),
                  class = "data.frame")
     } else { # Return a matrix
       splitfun2 <- if(multi) function(y) do.call(rbind, .mapply(FUN, c(list(gsplit(y, g)), asl), mord)) else
@@ -286,7 +286,7 @@ BY.matrix <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reorder =
         reorder <- FALSE
       }
     } else { # function doesn't assign names, different options.
-      res1 <- unlist(res1, FALSE, FALSE)
+      res1 <- funlist(res1)
       rn <- if(length(res1) == g[[1L]]) GRPnames(g) else
             if(length(res1) == n && (reorder || isTRUE(g$ordered[2L]))) dn[[1L]] else NULL
     }
@@ -310,7 +310,7 @@ BY.matrix <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reorder =
       if(nrow(res) != n || !(reorder || isTRUE(g$ordered[2L]))) dn <- list(NULL, dn[[2L]])
 
     } else { # Return data frame
-      rn <- if(length(res[[1L]]) == n && (reorder || isTRUE(g$ordered[2L]))) dn[[1L]] else NULL
+      rn <- if(.Call(C_fnrow, res) == n && (reorder || isTRUE(g$ordered[2L]))) dn[[1L]] else NULL
     }
 
   }
@@ -328,7 +328,7 @@ BY.matrix <- function(x, g, FUN, ..., use.g.names = TRUE, sort = TRUE, reorder =
     ax[["dimnames"]] <- dn
   } else { # Returning a data.frame
     ax <- list(names = dn[[2L]],
-               row.names = if(length(rn)) rn else .set_row_names(length(res[[1L]])),
+               row.names = if(length(rn)) rn else .set_row_names(.Call(C_fnrow, res)),
                class = "data.frame")
   }
 
@@ -373,4 +373,5 @@ BY.grouped_df <- function(x, FUN, ..., reorder = TRUE, keep.group_vars = TRUE, u
   condalcSA(c(g[[4L]], res), ar, any(ar$class == "data.table"))
 }
 
-
+BY.zoo <- function(x, ...) if(is.matrix(x)) BY.matrix(x, ...) else BY.default(x, ...)
+BY.units <- BY.zoo # return = "same" preserves attributes by default
