@@ -68,7 +68,7 @@ static const R_CallMethodDef CallEntries[] = {
   {"Cpp_fvarsdl", (DL_FUNC) &_collapse_fvarsdlCpp, 9},
   {"Cpp_mrtl", (DL_FUNC) &_collapse_mrtl, 3},
   {"Cpp_mctl", (DL_FUNC) &_collapse_mctl, 3},
-  {"Cpp_psmat", (DL_FUNC) &_collapse_psmatCpp, 4},
+  {"Cpp_psmat", (DL_FUNC) &_collapse_psmatCpp, 5},
   {"Cpp_qF", (DL_FUNC) &_collapse_qFCpp, 5},
   {"Cpp_sortunique", (DL_FUNC) &_collapse_sortuniqueCpp, 1},
   {"Cpp_fdroplevels", (DL_FUNC) &_collapse_fdroplevelsCpp, 2},
@@ -88,6 +88,9 @@ static const R_CallMethodDef CallEntries[] = {
   {"C_gsplit", (DL_FUNC) &gsplit, 3},
   {"C_greorder", (DL_FUNC) &greorder, 2},
   {"C_lassign", (DL_FUNC) &lassign, 4},
+  {"C_gwhich_first", (DL_FUNC) &gwhich_first, 3},
+  {"C_gslice_multi", (DL_FUNC) &gslice_multi, 4},
+  {"C_funlist", (DL_FUNC) &funlist, 1},
   {"Cpp_seqid", (DL_FUNC) &_collapse_seqid, 7},
   {"Cpp_groupid", (DL_FUNC) &_collapse_groupid, 5},
   {"C_collapse_init", (DL_FUNC) &collapse_init, 1},
@@ -107,14 +110,16 @@ static const R_CallMethodDef CallEntries[] = {
   {"C_group", (DL_FUNC) &groupVec, 3},
   {"C_groupat", (DL_FUNC) &groupAtVec, 3},
   {"C_funique", (DL_FUNC) &funiqueC, 1},
+  {"C_fmatch", (DL_FUNC) &fmatchC, 5},
+  {"C_multi_match", (DL_FUNC) &multi_match, 2},
   {"C_radixsort", (DL_FUNC) &Cradixsort, 6},
   {"C_frankds", (DL_FUNC) &frankds, 4},
   {"C_pacf1", (DL_FUNC) &pacf1, 2},
   {"C_rbindlist", (DL_FUNC) &rbindlist, 4},
   {"C_setcolorder", (DL_FUNC) &setcolorder, 2},
   {"C_subsetCols", (DL_FUNC) &subsetCols, 3},
-  {"C_alloc", (DL_FUNC) &falloc, 2},
-  {"C_frange", (DL_FUNC) &frange, 2},
+  {"C_alloc", (DL_FUNC) &falloc, 3},
+  {"C_frange", (DL_FUNC) &frange, 3},
   {"C_fdist", (DL_FUNC) &fdist, 4},
   {"C_fnrow", (DL_FUNC) &fnrowC, 1},
   {"C_createeptr", (DL_FUNC) &createeptr, 1},
@@ -124,6 +129,13 @@ static const R_CallMethodDef CallEntries[] = {
   {"C_vecgcd", (DL_FUNC) &vecgcd, 1},
   {"C_issorted", (DL_FUNC) &Cissorted, 2},
   {"C_all_funs", (DL_FUNC) &all_funs, 1},
+  {"C_unlock_collapse_namespace", (DL_FUNC) &unlock_collapse_namespace, 1},
+  {"C_pivot_long", (DL_FUNC) &pivot_long, 3},
+  {"C_pivot_wide", (DL_FUNC) &pivot_wide, 7},
+  {"C_sort_merge_join", (DL_FUNC) &sort_merge_join, 4},
+  {"C_replace_outliers", (DL_FUNC) &replace_outliers, 5},
+  {"C_na_locf", (DL_FUNC) &na_locf, 2},
+  {"C_na_focb", (DL_FUNC) &na_focb, 2},
   // {"C_aschar", (DL_FUNC) &CasChar, 1},
   {"C_subsetDT", (DL_FUNC) &subsetDT, 4},
   {"C_subsetVector", (DL_FUNC) &subsetVector, 3},
@@ -151,12 +163,13 @@ void R_init_collapse(DllInfo *dll) {
      recommend you contact me if you want to use a C function in a package. I am always happy to answer questions
      regarding the arguments and use of certain C functions. */
 
-  // Funtions that fully operate on R vectors (SEXP)                           // Corresponding R function(s)
+  // Functions that fully operate on R vectors (SEXP)                           // Corresponding R function(s)
   R_RegisterCCallable("collapse", "cp_TRA", (DL_FUNC) &TRAC);                  // TRA.default()
   R_RegisterCCallable("collapse", "cp_setop", (DL_FUNC) &setop);               // setop()
   R_RegisterCCallable("collapse", "cp_range", (DL_FUNC) &frange);              // frange()
   R_RegisterCCallable("collapse", "cp_dist", (DL_FUNC) &fdist);                // fdist()
   R_RegisterCCallable("collapse", "cp_quantile", (DL_FUNC) &fquantileC);       // .quantile()
+  R_RegisterCCallable("collapse", "cp_match", (DL_FUNC) &fmatchC);             // fmatch()
   R_RegisterCCallable("collapse", "cp_group", (DL_FUNC) &groupVec);            // group(): main hash-based grouping function: for atomic vectors and data frames
   R_RegisterCCallable("collapse", "cp_group_at", (DL_FUNC) &groupAtVec);       // qG(.., sort = FALSE): same but only works with atomic vectors and has option to keep missing values
   R_RegisterCCallable("collapse", "cp_unique", (DL_FUNC) &funiqueC);           // funique.default()
