@@ -1,8 +1,6 @@
-#ifdef _OPENMP
-#include <omp.h>
-#endif
+#include "collapse_c.h" // Needs to be first because includes OpenMP
 #include "kit.h"
-#include "collapse_c.h"
+
 
 static double NEG_INF = -1.0/0.0;
 
@@ -20,8 +18,8 @@ int mode_int(const int *restrict px, const int *restrict po, const int l, const
     M *= 2;
     K++;
   }
-  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values
-  int *restrict n = (int*)Calloc(l, int); // Table to count frequency of values
+  int *restrict h = (int*)R_Calloc(M, int); // Table to save the hash values
+  int *restrict n = (int*)R_Calloc(l, int); // Table to count frequency of values
 
   if(sorted) {
     mode = px[0];
@@ -81,8 +79,8 @@ int mode_int(const int *restrict px, const int *restrict po, const int l, const
     }
   }
 
-  Free(h);
-  Free(n);
+  R_Free(h);
+  R_Free(n);
   return mode;
 }
 
@@ -99,8 +97,8 @@ int w_mode_int(const int *restrict px, const double *restrict pw, const int *res
     M *= 2;
     K++;
   }
-  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values
-  double *restrict sumw = (double*)Calloc(l, double); // Table to save each values sum of weights
+  int *restrict h = (int*)R_Calloc(M, int); // Table to save the hash values
+  double *restrict sumw = (double*)R_Calloc(l, double); // Table to save each values sum of weights
   double max = NEG_INF;
 
   if(sorted) {
@@ -164,8 +162,8 @@ int w_mode_int(const int *restrict px, const double *restrict pw, const int *res
     }
   }
 
-  Free(h);
-  Free(sumw);
+  R_Free(h);
+  R_Free(sumw);
   return mode;
 }
 
@@ -174,7 +172,7 @@ int mode_fct_logi(const int *restrict px, const int *restrict po, const int l, c
   if(l == 1) return sorted ? px[0] : px[po[0]-1];
   int val, mode, max = 1, nlevp = nlev + 1, i = 0, end = l-1,
     minm = ret == 1, nfirstm = ret > 0, lastm = ret == 3;
-  int *restrict n = (int*)Calloc(nlevp+1, int); // Table to count frequency of values
+  int *restrict n = (int*)R_Calloc(nlevp+1, int); // Table to count frequency of values
 
   if(sorted) {
     mode = px[0];
@@ -222,7 +220,7 @@ int mode_fct_logi(const int *restrict px, const int *restrict po, const int l, c
     }
   }
 
-  Free(n);
+  R_Free(n);
   return mode;
 }
 
@@ -233,7 +231,7 @@ int w_mode_fct_logi(const int *restrict px, const double *restrict pw, const int
   }
   int val, mode, nlevp = nlev + 1, i = 0, end = l-1,
     minm = ret == 1, nfirstm = ret > 0, lastm = ret == 3;
-  double *restrict sumw = (double*)Calloc(nlevp+1, double); // Table to save each values sum of weights
+  double *restrict sumw = (double*)R_Calloc(nlevp+1, double); // Table to save each values sum of weights
   double max = NEG_INF;
 
   if(sorted) {
@@ -287,7 +285,7 @@ int w_mode_fct_logi(const int *restrict px, const double *restrict pw, const int
     }
   }
 
-  Free(sumw);
+  R_Free(sumw);
   return mode;
 }
 
@@ -302,8 +300,8 @@ double mode_double(const double *restrict px, const int *restrict po, const int
     M *= 2;
     K++;
   }
-  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values
-  int *restrict n = (int*)Calloc(l, int); // Table to count frequency of values
+  int *restrict h = (int*)R_Calloc(M, int); // Table to save the hash values
+  int *restrict n = (int*)R_Calloc(l, int); // Table to count frequency of values
   double val, mode;
   union uno tpv;
 
@@ -313,7 +311,7 @@ double mode_double(const double *restrict px, const int *restrict po, const int
     for(; i < l; ++i) {
       val = px[i];
       if(ISNAN(val) && narm) continue;
-      tpv.d = val;
+      tpv.d = val + 0.0;
       id = HASH(tpv.u[0] + tpv.u[1], K);
       while(h[id]) {
         index = h[id]-1;
@@ -342,7 +340,7 @@ double mode_double(const double *restrict px, const int *restrict po, const int
     for(; i < l; ++i) {
       val = px[po[i]-1];
       if(ISNAN(val) && narm) continue;
-      tpv.d = val;
+      tpv.d = val + 0.0;
       id = HASH(tpv.u[0] + tpv.u[1], K);
       while(h[id]) {
         index = h[id]-1;
@@ -367,8 +365,8 @@ double mode_double(const double *restrict px, const int *restrict po, const int
     }
   }
 
-  Free(h);
-  Free(n);
+  R_Free(h);
+  R_Free(n);
   return mode;
 }
 
@@ -384,8 +382,8 @@ double w_mode_double(const double *restrict px, const double *restrict pw, const
     M *= 2;
     K++;
   }
-  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values
-  double *restrict sumw = (double*)Calloc(l, double); // Table to save each values sum of weights
+  int *restrict h = (int*)R_Calloc(M, int); // Table to save the hash values
+  double *restrict sumw = (double*)R_Calloc(l, double); // Table to save each values sum of weights
   double val, mode, max = NEG_INF;
   union uno tpv;
 
@@ -395,7 +393,7 @@ double w_mode_double(const double *restrict px, const double *restrict pw, const
     for(; i < l; ++i) {
       val = px[i];
       if(ISNAN(pw[i]) || (ISNAN(val) && narm)) continue;
-      tpv.d = val;
+      tpv.d = val + 0.0;
       id = HASH(tpv.u[0] + tpv.u[1], K);
       while(h[id]) {
         index = h[id]-1;
@@ -426,7 +424,7 @@ double w_mode_double(const double *restrict px, const double *restrict pw, const
       oi = po[i]-1;
       val = px[oi];
       if(ISNAN(pw[oi]) || (ISNAN(val) && narm)) continue;
-      tpv.d = val;
+      tpv.d = val + 0.0;
       id = HASH(tpv.u[0] + tpv.u[1], K);
       while(h[id]) {
         index = h[id]-1;
@@ -452,8 +450,8 @@ double w_mode_double(const double *restrict px, const double *restrict pw, const
     }
   }
 
-  Free(h);
-  Free(sumw);
+  R_Free(h);
+  R_Free(sumw);
   return mode;
 }
 
@@ -468,8 +466,8 @@ SEXP mode_string(const SEXP *restrict px, const int *restrict po, const int l, c
     M *= 2;
     K++;
   }
-  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values
-  int *restrict n = (int*)Calloc(l, int); // Table to count frequency of values
+  int *restrict h = (int*)R_Calloc(M, int); // Table to save the hash values
+  int *restrict n = (int*)R_Calloc(l, int); // Table to count frequency of values
   SEXP val, mode;
 
   if(sorted) {
@@ -478,7 +476,7 @@ SEXP mode_string(const SEXP *restrict px, const int *restrict po, const int l, c
     for(; i < l; ++i) {
       val = px[i];
       if(val == NA_STRING && narm) continue;
-      id = HASH(((intptr_t) val & 0xffffffff), K);
+      id = HASH(((uintptr_t) val & 0xffffffff), K);
       while(h[id]) {
         index = h[id]-1;
         if(px[index] == val) goto sbls;
@@ -506,7 +504,7 @@ SEXP mode_string(const SEXP *restrict px, const int *restrict po, const int l, c
     for(; i < l; ++i) {
       val = px[po[i]-1];
       if(val == NA_STRING && narm) continue;
-      id = HASH(((intptr_t) val & 0xffffffff), K);
+      id = HASH(((uintptr_t) val & 0xffffffff), K);
       while(h[id]) {
         index = h[id]-1;
         if(px[po[index]-1] == val) goto sbl;
@@ -530,8 +528,8 @@ SEXP mode_string(const SEXP *restrict px, const int *restrict po, const int l, c
     }
   }
 
-  Free(h);
-  Free(n);
+  R_Free(h);
+  R_Free(n);
   return mode;
 }
 
@@ -547,8 +545,8 @@ SEXP w_mode_string(const SEXP *restrict px, const double *restrict pw, const int
     M *= 2;
     K++;
   }
-  int *restrict h = (int*)Calloc(M, int); // Table to save the hash values
-  double *restrict sumw = (double*)Calloc(l, double); // Table to save each values sum of weights
+  int *restrict h = (int*)R_Calloc(M, int); // Table to save the hash values
+  double *restrict sumw = (double*)R_Calloc(l, double); // Table to save each values sum of weights
   double max = NEG_INF;
   SEXP val, mode;
 
@@ -558,7 +556,7 @@ SEXP w_mode_string(const SEXP *restrict px, const double *restrict pw, const int
     for(; i != l; ++i) {
       val = px[i];
       if(ISNAN(pw[i]) || (val == NA_STRING && narm)) continue;
-      id = HASH(((intptr_t) val & 0xffffffff), K);
+      id = HASH(((uintptr_t) val & 0xffffffff), K);
       while(h[id]) {
         index = h[id]-1;
         if(px[index] == val) goto sbls;
@@ -588,7 +586,7 @@ SEXP w_mode_string(const SEXP *restrict px, const double *restrict pw, const int
       oi = po[i]-1;
       val = px[oi];
       if(ISNAN(pw[oi]) || (val == NA_STRING && narm)) continue;
-      id = HASH(((intptr_t) val & 0xffffffff), K);
+      id = HASH(((uintptr_t) val & 0xffffffff), K);
       while(h[id]) {
         index = h[id]-1;
         if(px[po[index]-1] == val) goto sbl;
@@ -613,8 +611,8 @@ SEXP w_mode_string(const SEXP *restrict px, const double *restrict pw, const int
     }
   }
 
-  Free(h);
-  Free(sumw);
+  R_Free(h);
+  R_Free(sumw);
   return mode;
 }
 
@@ -639,7 +637,7 @@ SEXP mode_impl_plain(SEXP x, int narm, int ret) {
     case INTSXP:  return ScalarInteger(isFactor(x) ? mode_fct_logi(INTEGER(x), &l, l, nlevels(x), 1, narm, ret) :
                                     mode_int(INTEGER(x), &l, l, 1, narm, ret));
     case LGLSXP: return my_ScalarLogical(mode_fct_logi(LOGICAL(x), &l, l, 1, 1, narm, ret));
-    case STRSXP: return ScalarString(mode_string(STRING_PTR(x), &l, l, 1, narm, ret));
+    case STRSXP: return ScalarString(mode_string(SEXPPTR_RO(x), &l, l, 1, narm, ret));
     default: error("Not Supported SEXP Type: '%s'", type2char(TYPEOF(x)));
   }
 }
@@ -661,7 +659,7 @@ SEXP w_mode_impl_plain(SEXP x, double *pw, int narm, int ret) {
     case INTSXP:  return ScalarInteger(isFactor(x) ? w_mode_fct_logi(INTEGER(x), pw, &l, l, nlevels(x), 1, narm, ret) :
                                     w_mode_int(INTEGER(x), pw, &l, l, 1, narm, ret));
     case LGLSXP:  return my_ScalarLogical(w_mode_fct_logi(LOGICAL(x), pw, &l, l, 1, 1, narm, ret));
-    case STRSXP:  return ScalarString(w_mode_string(STRING_PTR(x), pw, &l, l, 1, narm, ret));
+    case STRSXP:  return ScalarString(w_mode_string(SEXPPTR_RO(x), pw, &l, l, 1, narm, ret));
     default: error("Not Supported SEXP Type: '%s'", type2char(TYPEOF(x)));
   }
 }
@@ -713,7 +711,8 @@ SEXP mode_g_impl(SEXP x, int ng, int *pgs, int *po, int *pst, int sorted, int na
         break;
       }
       case STRSXP: {
-        SEXP *px = STRING_PTR(x), *pres = STRING_PTR(res);
+        const SEXP *px = SEXPPTR_RO(x);
+        SEXP *pres = SEXPPTR(res);
         #pragma omp parallel for num_threads(nthreads)
         for(int gr = 0; gr < ng; ++gr)
           pres[gr] = pgs[gr] == 0 ? NA_STRING : mode_string(px + pst[gr]-1, po, pgs[gr], 1, narm, ret);
@@ -752,7 +751,8 @@ SEXP mode_g_impl(SEXP x, int ng, int *pgs, int *po, int *pst, int sorted, int na
         break;
       }
       case STRSXP: {
-        SEXP *px = STRING_PTR(x), *pres = STRING_PTR(res);
+        const SEXP *px = SEXPPTR_RO(x);
+        SEXP *pres = SEXPPTR(res);
         #pragma omp parallel for num_threads(nthreads)
         for(int gr = 0; gr < ng; ++gr)
           pres[gr] = pgs[gr] == 0 ? NA_STRING : mode_string(px, po + pst[gr]-1, pgs[gr], 0, narm, ret);
@@ -806,7 +806,8 @@ SEXP w_mode_g_impl(SEXP x, double *pw, int ng, int *pgs, int *po, int *pst, int
         break;
       }
       case STRSXP: {
-        SEXP *px = STRING_PTR(x), *pres = STRING_PTR(res);
+        const SEXP *px = SEXPPTR_RO(x);
+        SEXP *pres = SEXPPTR(res);
         #pragma omp parallel for num_threads(nthreads)
         for(int gr = 0; gr < ng; ++gr)
           pres[gr] = pgs[gr] == 0 ? NA_STRING : w_mode_string(px + pst[gr]-1, pw + pst[gr]-1, po, pgs[gr], 1, narm, ret);
@@ -845,7 +846,8 @@ SEXP w_mode_g_impl(SEXP x, double *pw, int ng, int *pgs, int *po, int *pst, int
         break;
       }
       case STRSXP: {
-        SEXP *px = STRING_PTR(x), *pres = STRING_PTR(res);
+        const SEXP *px = SEXPPTR_RO(x);
+        SEXP *pres = SEXPPTR(res);
         #pragma omp parallel for num_threads(nthreads)
         for(int gr = 0; gr < ng; ++gr)
           pres[gr] = pgs[gr] == 0 ? NA_STRING : w_mode_string(px, pw, po + pst[gr]-1, pgs[gr], 0, narm, ret);
@@ -882,7 +884,7 @@ SEXP fmodeC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rret, SEXP Rnthreads) {
     return w_mode_impl(x, pw, asLogical(Rnarm), asInteger(Rret));
   }
   if(TYPEOF(g) != VECSXP || !inherits(g, "GRP")) error("g needs to be an object of class 'GRP', see ?GRP");
-  const SEXP *restrict pg = SEXPPTR(g), o = pg[6];
+  const SEXP *restrict pg = SEXPPTR_RO(g), o = pg[6];
   int sorted = LOGICAL(pg[5])[1] == 1, ng = INTEGER(pg[0])[0], *restrict pgs = INTEGER(pg[2]), *restrict po, *restrict pst, nthreads = asInteger(Rnthreads);
   if(l != length(pg[1])) error("length(g) must match length(x)");
   if(isNull(o)) {
@@ -891,14 +893,14 @@ SEXP fmodeC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rret, SEXP Rnthreads) {
     pst = cgs + 1;
     if(sorted) po = &l;
     else {
-      int *restrict count = (int *) Calloc(ng+1, int);
+      int *restrict count = (int *) R_Calloc(ng+1, int);
       po = (int *) R_alloc(l, sizeof(int)); --po;
       for(int i = 0; i != l; ++i) po[cgs[pgv[i]] + count[pgv[i]]++] = i+1;
-      ++po; Free(count);
+      ++po; R_Free(count);
     }
   } else {
     po = INTEGER(o);
-    pst = INTEGER(getAttrib(o, install("starts")));
+    pst = INTEGER(getAttrib(o, sym_starts));
   }
   // if(nullw) return mode_g_impl(x, ng, pgs, po, pst, sorted, asLogical(Rnarm), asInteger(Rret), asInteger(Rnthreads));
   // if(TYPEOF(w) != REALSXP) UNPROTECT(nprotect);
@@ -919,11 +921,12 @@ SEXP fmodelC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rret, SEXP Rnthreads) {
     narm = asLogical(Rnarm), ret = asInteger(Rret), nthreads = asInteger(Rnthreads);
   if(l < 1) return x;
   if(nthreads > max_threads) nthreads = max_threads;
-  SEXP out = PROTECT(allocVector(VECSXP, l)), *restrict pout = SEXPPTR(out), *restrict px = SEXPPTR(x);
+  SEXP out = PROTECT(allocVector(VECSXP, l)), *restrict pout = SEXPPTR(out);
+  const SEXP *restrict px = SEXPPTR_RO(x);
   if(nullg && nthreads > l) nthreads = l;
   if(nullg && nullw) {
     if(nthreads <= 1) {
-      for(int j = 0; j != l; ++j) pout[j] = mode_impl(px[j], narm, ret);
+      for(int j = 0; j != l; ++j) SET_VECTOR_ELT(out, j, mode_impl(px[j], narm, ret));
     } else {
       #pragma omp parallel for num_threads(nthreads)
       for(int j = 0; j < l; ++j) pout[j] = mode_impl_plain(px[j], narm, ret);
@@ -942,7 +945,7 @@ SEXP fmodelC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rret, SEXP Rnthreads) {
     }
     if(nullg) {
       if(nthreads <= 1) {
-        for(int j = 0; j != l; ++j) pout[j] = w_mode_impl(px[j], pw, narm, ret);
+        for(int j = 0; j != l; ++j) SET_VECTOR_ELT(out, j, w_mode_impl(px[j], pw, narm, ret));
       } else {
         #pragma omp parallel for num_threads(nthreads)
         for(int j = 0; j < l; ++j) pout[j] = w_mode_impl_plain(px[j], pw, narm, ret);
@@ -950,7 +953,7 @@ SEXP fmodelC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rret, SEXP Rnthreads) {
       }
     } else {
       if(TYPEOF(g) != VECSXP || !inherits(g, "GRP")) error("g needs to be an object of class 'GRP', see ?GRP");
-      const SEXP *restrict pg = SEXPPTR(g), o = pg[6];
+      const SEXP *restrict pg = SEXPPTR_RO(g), o = pg[6];
       ng = INTEGER(pg[0])[0];
       int sorted = LOGICAL(pg[5])[1] == 1, *restrict pgs = INTEGER(pg[2]), *restrict po, *restrict pst;
       if(nrx != length(pg[1])) error("length(g) must match nrow(x)");
@@ -960,19 +963,19 @@ SEXP fmodelC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rret, SEXP Rnthreads) {
         pst = cgs + 1;
         if(sorted) po = &l;
         else {
-          int *restrict count = (int *) Calloc(ng+1, int);
+          int *restrict count = (int *) R_Calloc(ng+1, int);
           po = (int *) R_alloc(nrx, sizeof(int)); --po;
           for(int i = 0; i != nrx; ++i) po[cgs[pgv[i]] + count[pgv[i]]++] = i+1;
-          ++po; Free(count);
+          ++po; R_Free(count);
         }
       } else {
         po = INTEGER(o);
-        pst = INTEGER(getAttrib(o, install("starts")));
+        pst = INTEGER(getAttrib(o, sym_starts));
       }
       if(nullw) { // Parallelism at sub-column level
-        for(int j = 0; j < l; ++j) pout[j] = mode_g_impl(px[j], ng, pgs, po, pst, sorted, narm, ret, nthreads);
+        for(int j = 0; j < l; ++j) SET_VECTOR_ELT(out, j, mode_g_impl(px[j], ng, pgs, po, pst, sorted, narm, ret, nthreads));
       } else { // Parallelism at sub-column level
-        for(int j = 0; j < l; ++j) pout[j] = w_mode_g_impl(px[j], pw, ng, pgs, po, pst, sorted, narm, ret, nthreads);
+        for(int j = 0; j < l; ++j) SET_VECTOR_ELT(out, j, w_mode_g_impl(px[j], pw, ng, pgs, po, pst, sorted, narm, ret, nthreads));
       }
     }
   }
@@ -1040,7 +1043,8 @@ SEXP fmodemC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rret, SEXP Rnt
         break;
       }
       case STRSXP: {
-        SEXP *px = STRING_PTR(x), *restrict pres = STRING_PTR(res);
+        const SEXP *px = SEXPPTR_RO(x);
+        SEXP *restrict pres = SEXPPTR(res);
         if(nullw) {
           #pragma omp parallel for num_threads(nthreads)
           for(int j = 0; j < col; ++j) pres[j] = mode_string(px + j*l, &l, l, 1, narm, ret);
@@ -1060,7 +1064,7 @@ SEXP fmodemC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rret, SEXP Rnt
 
   // With groups
   if(TYPEOF(g) != VECSXP || !inherits(g, "GRP")) error("g needs to be an object of class 'GRP', see ?GRP");
-  const SEXP *restrict pg = SEXPPTR(g), o = pg[6];
+  const SEXP *restrict pg = SEXPPTR_RO(g), o = pg[6];
   int sorted = LOGICAL(pg[5])[1] == 1, ng = INTEGER(pg[0])[0], *restrict pgs = INTEGER(pg[2]), *restrict po, *restrict pst, gl = length(pg[1]);
   if(l != gl) error("length(g) must match nrow(x)");
   SEXP res = PROTECT(allocVector(tx, ng * col));
@@ -1071,14 +1075,14 @@ SEXP fmodemC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rret, SEXP Rnt
     pst = cgs + 1;
     if(sorted) po = &l;
     else {
-      int *restrict count = (int *) Calloc(ng+1, int);
+      int *restrict count = (int *) R_Calloc(ng+1, int);
       po = (int *) R_alloc(l, sizeof(int)); --po;
       for(int i = 0; i != l; ++i) po[cgs[pgv[i]] + count[pgv[i]]++] = i+1;
-      ++po; Free(count);
+      ++po; R_Free(count);
     }
   } else {
     po = INTEGER(o);
-    pst = INTEGER(getAttrib(o, install("starts")));
+    pst = INTEGER(getAttrib(o, sym_starts));
   }
 
   if(sorted) { // Sorted
@@ -1137,19 +1141,20 @@ SEXP fmodemC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rret, SEXP Rnt
         break;
       }
       case STRSXP: {
-        SEXP *px = STRING_PTR(x), *restrict pres = STRING_PTR(res);
+        const SEXP *px = SEXPPTR_RO(x);
+        SEXP *restrict pres = SEXPPTR(res);
         if(nullw) {
           #pragma omp parallel for num_threads(nthreads)
           for(int j = 0; j < col; ++j) {
             int jng = j * ng;
-            SEXP *pxj = px + j * l;
+            const SEXP *pxj = px + j * l;
             for(int gr = 0; gr < ng; ++gr) pres[jng + gr] = pgs[gr] == 0 ? NA_STRING : mode_string(pxj + pst[gr]-1, po, pgs[gr], 1, narm, ret);
           }
         } else {
           #pragma omp parallel for num_threads(nthreads)
           for(int j = 0; j < col; ++j) {
             int jng = j * ng;
-            SEXP *pxj = px + j * l;
+            const SEXP *pxj = px + j * l;
             for(int gr = 0; gr < ng; ++gr) pres[jng + gr] = pgs[gr] == 0 ? NA_STRING : w_mode_string(pxj + pst[gr]-1, pw + pst[gr]-1, po, pgs[gr], 1, narm, ret);
           }
         }
@@ -1213,19 +1218,20 @@ SEXP fmodemC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rret, SEXP Rnt
         break;
       }
       case STRSXP: {
-        SEXP *px = STRING_PTR(x), *restrict pres = STRING_PTR(res);
+        const SEXP *px = SEXPPTR_RO(x);
+        SEXP *restrict pres = SEXPPTR(res);
         if(nullw) {
           #pragma omp parallel for num_threads(nthreads)
           for(int j = 0; j < col; ++j) {
             int jng = j * ng;
-            SEXP *pxj = px + j * l;
+            const SEXP *pxj = px + j * l;
             for(int gr = 0; gr < ng; ++gr) pres[jng + gr] = pgs[gr] == 0 ? NA_STRING : mode_string(pxj, po + pst[gr]-1, pgs[gr], 0, narm, ret);
           }
         } else {
           #pragma omp parallel for num_threads(nthreads)
           for(int j = 0; j < col; ++j) {
             int jng = j * ng;
-            SEXP *pxj = px + j * l;
+            const SEXP *pxj = px + j * l;
             for(int gr = 0; gr < ng; ++gr) pres[jng + gr] = pgs[gr] == 0 ? NA_STRING : w_mode_string(pxj, pw, po + pst[gr]-1, pgs[gr], 0, narm, ret);
           }
         }
