@@ -1,4 +1,5 @@
 #include "collapse_c.h"
+#include "data.table.h"
 
 SEXP Cna_rm(SEXP x) {
   const int n = LENGTH(x);
@@ -31,11 +32,11 @@ SEXP Cna_rm(SEXP x) {
     return out;
   }
   case STRSXP: {
-    const SEXP *xd = STRING_PTR(x);
+    const SEXP *xd = SEXPPTR_RO(x);
     for (int i = 0; i != n; ++i) if(xd[i] == NA_STRING) ++k;
     if(k == 0) return x;
     SEXP out = PROTECT(allocVector(STRSXP, n - k));
-    SEXP *pout = STRING_PTR(out);
+    SEXP *pout = SEXPPTR(out);
     k = 0;
     for (int i = 0; i != n; ++i) if(xd[i] != NA_STRING) pout[k++] = xd[i];
     copyMostAttrib(x, out);
@@ -60,8 +61,14 @@ SEXP Cna_rm(SEXP x) {
 
 // Helper function to find a single string in factor levels
 int fchmatch(SEXP x, SEXP val, int nomatch) {
-  const SEXP *px = STRING_PTR(x), v = asChar(val);
-  for(int i = 0, l = length(x); i != l; ++i) if(px[i] == v) return i + 1;
+  const SEXP *px = SEXPPTR_RO(PROTECT(coerceUtf8IfNeeded(x))), v = PROTECT(ENC2UTF8(asChar(val)));
+  for(int i = 0, l = length(x); i != l; ++i) {
+    if(px[i] == v) {
+      UNPROTECT(2);
+      return i + 1;
+    }
+  }
+  UNPROTECT(2);
   return nomatch;
 }
 
@@ -109,9 +116,10 @@ if(length(val) == n && n > 1) {
   }
   case STRSXP:
   {
-    const SEXP *px = STRING_PTR(x);
-    const SEXP *pv = STRING_PTR(val);
+    const SEXP *px = SEXPPTR_RO(PROTECT(coerceUtf8IfNeeded(x)));
+    const SEXP *pv = SEXPPTR_RO(PROTECT(coerceUtf8IfNeeded(val)));
     WHICHVLOOPLX
+    UNPROTECT(2);
     break;
   }
   case RAWSXP :
@@ -155,10 +163,10 @@ if(length(val) == n && n > 1) {
   }
   case STRSXP:
   {
-    const SEXP *px = STRING_PTR(x);
-    const SEXP v = PROTECT(asChar(val));
+    const SEXP *px = SEXPPTR_RO(PROTECT(coerceUtf8IfNeeded(x)));
+    const SEXP v = PROTECT(ENC2UTF8(asChar(val)));
     WHICHVLOOP
-    UNPROTECT(1);
+    UNPROTECT(2);
     break;
   }
   case RAWSXP :
@@ -216,9 +224,27 @@ case REALSXP:
 }
 case STRSXP:
 {
-  const SEXP *px = STRING_PTR(x);
-  const SEXP v = asChar(val);
-  ALLANYVLOOP
+  const SEXP *px = SEXPPTR_RO(PROTECT(coerceUtf8IfNeeded(x)));
+  const SEXP v = PROTECT(ENC2UTF8(asChar(val)));
+  if(all) {
+    for(int i = 0; i != n; ++i) {
+      if(px[i] != v) {
+        UNPROTECT(2);
+        return ScalarLogical(0);
+      }
+    }
+    UNPROTECT(2);
+    return ScalarLogical(1);
+  } else {
+    for(int i = 0; i != n; ++i) {
+      if(px[i] == v) {
+        UNPROTECT(2);
+        return ScalarLogical(1);
+      }
+    }
+    UNPROTECT(2);
+    return ScalarLogical(0);
+  }
   break;
 }
 case RAWSXP :
@@ -387,7 +413,7 @@ SEXP setcopyv(SEXP x, SEXP val, SEXP rep, SEXP Rinvert, SEXP Rset, SEXP Rind1) {
   }
   case STRSXP:
   {
-    SEXP *restrict px = set ? STRING_PTR(x) : STRING_PTR(ans);
+    SEXP *restrict px = set ? SEXPPTR(x) : SEXPPTR(ans);
     if(lv == 1 && ind1 == 0) {
       const SEXP v = PROTECT(asChar(val));
       if(lr == 1) {
@@ -395,7 +421,7 @@ SEXP setcopyv(SEXP x, SEXP val, SEXP rep, SEXP Rinvert, SEXP Rset, SEXP Rind1) {
         setcopyvLOOP(r)
         UNPROTECT(1);
       } else {
-        const SEXP *restrict pr = STRING_PTR(rep);
+        const SEXP *restrict pr = SEXPPTR_RO(rep);
         setcopyvLOOP(pr[i])
       }
       UNPROTECT(1);
@@ -406,7 +432,7 @@ SEXP setcopyv(SEXP x, SEXP val, SEXP rep, SEXP Rinvert, SEXP Rset, SEXP Rind1) {
         setcopyvLOOPLVEC1
         UNPROTECT(1);
       } else {
-        const SEXP *restrict pr = STRING_PTR(rep);
+        const SEXP *restrict pr = SEXPPTR_RO(rep);
         setcopyvLOOPLVEC
       }
     }
@@ -426,6 +452,31 @@ SEXP setcopyv(SEXP x, SEXP val, SEXP rep, SEXP Rinvert, SEXP Rset, SEXP Rind1) {
       const SEXP *restrict pr = SEXPPTR_RO(rep);
       setcopyvLOOPLVEC
     }
+    // // TODO: Probably not needed for now. Can set xlist = TRUE
+    // SEXP tmp = set ? x : ans;
+    // if(lr == 1) {
+    //   const SEXP r = VECTOR_ELT(rep, 0);
+    //   if(tv == INTSXP) {
+    //     for(int i = 0; i < lv; ++i) SET_VECTOR_ELT(tmp, pv[i]-1, r);
+    //   } else if(invert == 0) {
+    //     for(int i = 0; i != n; ++i) if(pv[i] > 0) SET_VECTOR_ELT(tmp, i, r);
+    //   } else {
+    //     for(int i = 0; i != n; ++i) if(pv[i] == 0) SET_VECTOR_ELT(tmp, i, r);
+    //   }
+    // } else {
+    //   const SEXP *restrict pr = SEXPPTR_RO(rep);
+    //   if(tv == INTSXP) {
+    //     if(lr == n) {
+    //       for(int i = 0; i < lv; ++i) SET_VECTOR_ELT(tmp, pv[i]-1, pr[pv[i]-1]);
+    //     } else {
+    //       for(int i = 0; i < lv; ++i) SET_VECTOR_ELT(tmp, pv[i]-1, pr[i]);
+    //     }
+    //   } else if(invert == 0) {
+    //     for(int i = 0; i != n; ++i) if(pv[i] > 0) SET_VECTOR_ELT(tmp, i, pr[i]);
+    //   } else {
+    //     for(int i = 0; i != n; ++i) if(pv[i] == 0) SET_VECTOR_ELT(tmp, i, pr[i]);
+    //   }
+    //
     break;
   }
   case RAWSXP:
@@ -760,7 +811,7 @@ SEXP na_locf(SEXP x, SEXP Rset) {
   }
   case STRSXP:
   {
-    SEXP *data = STRING_PTR(x);
+    SEXP *data = SEXPPTR(x);
     SEXP last = data[0];
     for (int i = 0; i < n; i++) {
       if (data[i] == NA_STRING) {
@@ -826,7 +877,7 @@ SEXP na_focb(SEXP x, SEXP Rset) {
   }
   case STRSXP:
   {
-    SEXP *data = STRING_PTR(x);
+    SEXP *data = SEXPPTR(x);
     SEXP last = data[0];
     for (int i = n; i--; ) {
       if (data[i] == NA_STRING) {
@@ -872,42 +923,27 @@ SEXP vtypes(SEXP x, SEXP isnum) {
   case 1: // Numeric variables: do_is with op = 100: https://github.com/wch/r-source/blob/2b0818a47199a0b64b6aa9b9f0e53a1e886e8e95/src/main/coerce.c
           // See also DispatchOrEval in https://github.com/wch/r-source/blob/trunk/src/main/eval.c
     {
-    if(inherits(x, "indexed_frame")) { // NOT pdata.frame!! because columns in pdata.frame only become pseries when extracted from the frame
-      for(int i = 0, tci, tnum, is_num; i != n; ++i) {
-        is_num = 0;
-        tci = TYPEOF(px[i]);
-        tnum = tci == INTSXP || tci == REALSXP;
-        if(tnum) is_num = inherits(px[i], "integer") || inherits(px[i], "numeric") || inherits(px[i], "ts") || inherits(px[i], "units");
-        pans[i] = tnum && is_num;
-      }
-      // for(int i = 0; i != n; ++i) {
-      //   int tci = TYPEOF(px[i]);
-      //   pans[i] = (tci == INTSXP && inherits(px[i], "integer")) || (tci == REALSXP && inherits(px[i], "numeric")); // length(getAttrib(ci, R_ClassSymbol)) <= 2;
-      // }
-    } else {
-      for(int i = 0, tci, tnum, is_num; i != n; ++i) {
-        // pans[i] = isNumeric(px[i]) && !isLogical(px[i]); // Date is numeric, from: https://github.com/wch/r-source/blob/2b0818a47199a0b64b6aa9b9f0e53a1e886e8e95/src/main/coerce.c
-        tci = TYPEOF(px[i]);
-        tnum = tci == INTSXP || tci == REALSXP;
-        is_num = tnum && OBJECT(px[i]) == 0;
-        if(tnum && !is_num) is_num = inherits(px[i], "ts") || inherits(px[i], "units");
-        pans[i] = is_num;
-      }
+    for(int i = 0, tci, tnum; i != n; ++i) {
+      // pans[i] = isNumeric(px[i]) && !isLogical(px[i]); // Date is numeric, from: https://github.com/wch/r-source/blob/2b0818a47199a0b64b6aa9b9f0e53a1e886e8e95/src/main/coerce.c
+      tci = TYPEOF(px[i]);
+      tnum = tci == INTSXP || tci == REALSXP;
+      if(tnum && isObject(px[i])) tnum = !(inherits(px[i], "factor") || inherits(px[i], "Date") || inherits(px[i], "POSIXct") || inherits(px[i], "yearmon") || inherits(px[i], "yearqtr"));
+      pans[i] = tnum;
     }
-    SET_TYPEOF(ans, LGLSXP);
+    SETTOF(ans, LGLSXP);
     break;
     }
   case 2: // is.factor
     for(int i = 0; i != n; ++i) pans[i] = (int)isFactor(px[i]);
-    SET_TYPEOF(ans, LGLSXP);
+    SETTOF(ans, LGLSXP);
     break;
   case 3: // is.list, needed for list processing functions
     for(int i = 0; i != n; ++i) pans[i] = TYPEOF(px[i]) == VECSXP;
-    SET_TYPEOF(ans, LGLSXP);
+    SETTOF(ans, LGLSXP);
     break;
   case 4: // is.sublist, needed for list processing functions
-    for(int i = 0; i != n; ++i) pans[i] = TYPEOF(px[i]) == VECSXP && !isFrame(px[i]);
-    SET_TYPEOF(ans, LGLSXP);
+    for(int i = 0; i != n; ++i) pans[i] = TYPEOF(px[i]) == VECSXP && !inherits(px[i], "data.frame");
+    SETTOF(ans, LGLSXP);
     break;
   case 7: // is.atomic(x), needed in atomic_elem()
     // is.atomic: do_is with op = 200:  https://github.com/wch/r-source/blob/9f9033e193071f256e21a181cb053cba983ed4a9/src/main/coerce.c
@@ -927,7 +963,7 @@ SEXP vtypes(SEXP x, SEXP isnum) {
         pans[i] = 0;
       }
     }
-    SET_TYPEOF(ans, LGLSXP);
+    SETTOF(ans, LGLSXP);
     break;
   case 5: // is.atomic(x) || is.list(x), needed in reg_elem() and irreg_elem()
     for(int i = 0; i != n; ++i) {
@@ -949,7 +985,7 @@ SEXP vtypes(SEXP x, SEXP isnum) {
         pans[i] = 0;
       }
     }
-    SET_TYPEOF(ans, LGLSXP);
+    SETTOF(ans, LGLSXP);
     break;
   case 6:
     // Faster object type identification, needed in unlist2d:
@@ -959,7 +995,7 @@ SEXP vtypes(SEXP x, SEXP isnum) {
         pans[i] = 1;
       else switch(TYPEOF(px[i])) {
            case VECSXP:
-             pans[i] = isFrame(px[i]) ? 2 : 0;
+             pans[i] = inherits(px[i], "data.frame") ? 2 : 0;
              break;
            case NILSXP: /* NULL is atomic (S compatibly), but not in isVectorAtomic(.) */
            case CHARSXP:
