@@ -1,6 +1,6 @@
 context("Misc")
 
-if(!is.null(attributes(identical(FALSE, TRUE)))) stop("OECD label issue")
+
 
 # rm(list = ls())
 set.seed(101)
@@ -10,6 +10,7 @@ F <- getNamespace("collapse")$F
 test_that("descr, pwcor, pwcov, pwnobs", {
 
   expect_visible(descr(wlddev))
+  expect_equal(lapply(wlddev, descr) %>% get_elem("i") %>% unattrib(), unattrib(descr(wlddev)))
   expect_visible(as.data.frame(descr(wlddev)))
   expect_output(print(descr(wlddev)))
   expect_visible(descr(GGDC10S))
@@ -100,10 +101,10 @@ test_that("adding and removing stubs works", {
   expect_identical(rm_stub(names(iris), "Width", regex = TRUE)[4], "Petal.")
 })
 
-test_that("deep matrix dispatch works well", {
+test_that("zoo dispatch works well", {
+  skip_if_not_installed("zoo")
 
-  tsm <- EuStockMarkets
-  class(tsm) <- setdiff(class(tsm), "matrix")
+  tsm <- zoo::as.zoo(EuStockMarkets)
   set.seed(101)
   f <- qF(sample.int(5, nrow(tsm), TRUE))
   NCOL2 <- function(x) if(length(d <- dim(x)) > 1L) d[2L] else length(x)
@@ -120,6 +121,32 @@ test_that("deep matrix dispatch works well", {
 
 })
 
+test_that("units support works well", {
+
+  v = abs(rnorm(5))
+  m = abs(matrix(rnorm(25), 5))
+  g = qF(c(1,1,2,3,3))
+  attributes(v) <- list(units = structure(list(numerator = "m", denominator = character(0)), class = "symbolic_units"), class = "units")
+  attributes(m) <- list(dim = c(5L, 5L), units = structure(list(numerator = "m", denominator = character(0)), class = "symbolic_units"), class = "units")
+
+  for (f in setdiff(c(.FAST_FUN, .OPERATOR_FUN), c("fnobs", "fndistinct", "F"))) {
+    # print(f)
+    FUN = match.fun(f)
+    if (!startsWith(f, "fhd") && !startsWith(f, "HD")) {
+      expect_true(inherits(FUN(v), "units"))
+      expect_true(inherits(FUN(m), "units"))
+    }
+    if (f %in% c("fnth","flag","L","fdiff","D","Dlog", "fgrowth","G")) {
+      expect_true(inherits(FUN(v, g = g), "units"))
+      expect_true(inherits(FUN(m, g = g), "units"))
+    } else {
+      expect_true(inherits(FUN(v, g), "units"))
+      expect_true(inherits(FUN(m, g), "units"))
+    }
+  }
+
+})
+
 m <- qM(mtcars)
 v <- mtcars$mpg
 f <- qF(mtcars$cyl)
@@ -229,7 +256,7 @@ test_that("fast functions give appropriate warnings", {
 })
 
 test_that("fselect and fsubset cannot easily be confuesed", {
-  expect_error(suppressWarnings(fsubset(mtcars, mpg:vs, wt)))
+  # expect_error(suppressWarnings(fsubset(mtcars, mpg:vs, wt)))
   expect_error(fselect(mtcars, mpg == 1))
 })
 
@@ -355,3 +382,23 @@ test_that("fdist works properly", {
     set_collapse(oldopts)
   }
 })
+
+test_that("rowbind", {
+  expect_equal(rowbind(mtcars, mtcars), setRownames(rbind(mtcars, mtcars)))
+  expect_equal(rowbind(list(mtcars, mtcars)), setRownames(rbind(mtcars, mtcars)))
+  expect_equal(rowbind(mtcars, mtcars), unlist2d(list(mtcars, mtcars), idcols = FALSE))
+  expect_equal(rowbind(mtcars, mtcars, idcol = "id"), unlist2d(list(mtcars, mtcars), idcols = "id"))
+  expect_equal(rowbind(mtcars, mtcars, row.names = "car"), unlist2d(list(mtcars, mtcars), idcols = FALSE, row.names = "car"))
+  expect_equal(rowbind(mtcars, mtcars, idcol = "id", row.names = "car"), unlist2d(list(mtcars, mtcars), idcols = "id", row.names = "car"))
+  expect_equal(rowbind(a = mtcars, b = mtcars, idcol = "id"), unlist2d(list(a = mtcars, b = mtcars), idcols = "id", id.factor = TRUE))
+  expect_equal(rowbind(a = mtcars, b = mtcars, idcol = "id", id.factor = FALSE), unlist2d(list(a = mtcars, b = mtcars), idcols = "id"))
+})
+
+if (requireNamespace("bit64", quietly = TRUE)) test_that("rowbind + integer64", {
+  # https://github.com/fastverse/collapse/issues/697
+  x <- data.frame(a = bit64::as.integer64(1))
+  xi <- data.frame(a = 1L)
+  xd <- data.frame(a = 1)
+  expect_equal(rowbind(x, xi), setRownames(rbind(x, x)))
+  expect_equal(rowbind(x, xd), setRownames(rbind(x, x)))
+})
