@@ -1,6 +1,6 @@
 context("miscellaneous issues")
 
-if(!is.null(attributes(identical(FALSE, TRUE)))) stop("OECD label issue")
+
 
 # rm(list = ls())
 
@@ -125,9 +125,9 @@ test_that("Using a factor with unused levels does not pose a problem to statisti
     }
 })
 
-if(requireNamespace("magrittr", quietly = TRUE)) {
-library(magrittr)
 test_that("Testing grouped_df methods", {
+  skip_if_not_installed("magrittr")
+  library(magrittr)
   for(sortg in c(TRUE, FALSE)) {
     for(retgrp in c(TRUE, FALSE)) {
       gdf <- wlddev %>% fsubset(year > 1990, region, income, PCGDP:ODA) %>% fgroup_by(region, income, return.groups = retgrp, sort = sortg)
@@ -219,19 +219,17 @@ test_that("Testing grouped_df methods", {
     }
   }
 })
-}
 
 # Also better not run on CRAN...
 test_that("0-length vectors give expected output", {
   funs <- .c(fsum, fprod, fmean, fmedian, fmin, fmax, fnth, fcumsum, fbetween, fwithin, fscale)
   for(i in funs) {
     FUN <- match.fun(i)
-    if(i %!in% .c(fsum, fmin, fmax, fcumsum, fprod, fmean, fmedian, fnth)) {
+    if(i %!in% .c(fsum, fmin, fmax, fcumsum)) {
       expect_true(all_identical(FUN(numeric(0)), FUN(integer(0)), numeric(0)))
     } else {
       expect_identical(FUN(numeric(0)), numeric(0))
-      if(i %in% .c(fmean, fprod, fnth, fmedian)) expect_identical(FUN(integer(0)), NA_real_)
-      else expect_identical(FUN(integer(0)), integer(0))
+      expect_identical(FUN(integer(0)), integer(0))
     }
   }
   funs <- .c(fmode, ffirst, flast)
@@ -279,6 +277,7 @@ gf <- as_factor_qG(g)
 funs <- grep("hd|log", c(.FAST_FUN, .OPERATOR_FUN), ignore.case = TRUE, invert = TRUE, value = TRUE)
 
 test_that("functions work on plain matrices", {
+  F <- getNamespace("collapse")$F
   for(i in funs) {
     expect_visible(match.fun(i)(X))
     expect_visible(match.fun(i)(X, g = g))
@@ -291,6 +290,7 @@ test_that("functions work on plain matrices", {
 Xl <- mctl(X)
 
 test_that("functions work on plain lists", {
+  F <- getNamespace("collapse")$F
   for(i in funs) {
     expect_visible(match.fun(i)(Xl))
     expect_visible(match.fun(i)(Xl, g = g, by = g))
@@ -321,10 +321,10 @@ test_that("functions using welfords method properly deal with zero weights", {
     expect_equal(unattrib(fsd(x = c(2, 1, 3), g = g, w = c(0, 1, 1), na.rm = FALSE)), sqrt(2))
     expect_equal(unattrib(fscale(x = c(2, 1, 0), g = g, w = c(1, 1, 0), na.rm = TRUE)), (c(2, 1, 0)-1.5)/sqrt(0.5))
     expect_equal(unattrib(fscale(x = c(2, 1, 3), g = g, w = c(0, 1, 1), na.rm = FALSE)), (c(2, 1, 3)-2)/sqrt(2))
-    expect_equal(unattrib(qsu(x = c(2, 1, 0), g = g, w = c(1, 1, 0))), c(2, 1.5, sqrt(0.5), 1, 2))
-    expect_equal(unattrib(qsu(x = c(2, 1, 3), g = g, w = c(0, 1, 1))), c(2, 2, sqrt(2), 1, 3))
-    expect_equal(unattrib(qsu(x = c(2, 1, 0), g = g, w = c(1, 1, 0), higher = TRUE))[1:5], c(2, 1.5, sqrt(0.5), 1, 2))
-    expect_equal(unattrib(qsu(x = c(2, 1, 3), g = g, w = c(0, 1, 1), higher = TRUE))[1:5], c(2, 2, sqrt(2), 1, 3))
+    expect_equal(unattrib(qsu(x = c(2, 1, 0), g = g, w = c(1, 1, 0)))[-2L], c(2, 1.5, sqrt(0.5), 1, 2))
+    expect_equal(unattrib(qsu(x = c(2, 1, 3), g = g, w = c(0, 1, 1)))[-2L], c(2, 2, sqrt(2), 1, 3))
+    expect_equal(unattrib(qsu(x = c(2, 1, 0), g = g, w = c(1, 1, 0), higher = TRUE))[c(1L, 3:6)], c(2, 1.5, sqrt(0.5), 1, 2))
+    expect_equal(unattrib(qsu(x = c(2, 1, 3), g = g, w = c(0, 1, 1), higher = TRUE))[c(1L, 3:6)], c(2, 2, sqrt(2), 1, 3))
 
   }
 })
@@ -393,4 +393,114 @@ test_that("singleton groups are handled properly by all statistical functions",
 })
 
 
+test_that("functions work for data frames with zero rows", {
+  mtc0 <- qDF(mtcars)[NULL, ]
+  expect_equal(mtc0, funique(mtc0))
+  expect_equal(mtc0, funique(mtc0, sort = TRUE))
+  expect_equal(mtc0, roworderv(mtc0))
+  expect_visible(colorder(mtc0, mpg, hp))
+  expect_visible(GRP(mtc0))
+  expect_visible(fgroup_by(mtc0, cyl, vs, am))
+  expect_visible(GRP(mtc0, sort = FALSE))
+  expect_visible(fgroup_by(mtc0, cyl, vs, am, sort = FALSE))
+  expect_visible(fduplicated(mtc0))
+  expect_false(any_duplicated(mtc0))
+  expect_visible(fselect(mtc0, hp, carb))
+  expect_visible(get_vars(mtc0, 9:8))
+})
+
+test_that("issue with integer followed by NA #432", {
+    for (f in setdiff(.FAST_STAT_FUN, c("fvar", "fsd", "fnobs", "fndistinct"))) {
+      # if(!isTRUE(all.equal(match.fun(f)(c(10L, NA)), 10L))) print(f)
+      expect_equal(match.fun(f)(c(10L, NA)), 10L)
+      expect_equal(match.fun(f)(c(NA, 10L)), 10L)
+      expect_equal(match.fun(f)(c(10, NA)), 10)
+      expect_equal(match.fun(f)(c(NA, 10)), 10)
+      expect_equal(match.fun(f)(c(10L, NA), g = rep(1L, 2), use.g.names = FALSE), 10L)
+      expect_equal(match.fun(f)(c(NA, 10L), g = rep(1L, 2), use.g.names = FALSE), 10L)
+      expect_equal(match.fun(f)(c(10, NA), g = rep(1L, 2), use.g.names = FALSE), 10)
+      expect_equal(match.fun(f)(c(NA, 10), g = rep(1L, 2), use.g.names = FALSE), 10)
+      # na.rm = FALSE
+      if(f %!in% c("fmode", "ffirst")) expect_equal(match.fun(f)(c(10L, NA), na.rm = FALSE), NA_integer_)
+      if(f != "flast") expect_equal(match.fun(f)(c(NA, 10L), na.rm = FALSE), NA_integer_)
+      if(f %!in% c("fmode", "ffirst")) expect_equal(match.fun(f)(c(10, NA), na.rm = FALSE), NA_real_)
+      if(f != "flast") expect_equal(match.fun(f)(c(NA, 10), na.rm = FALSE), NA_real_)
+      # Some functions are optimized and don't check here
+      # expect_equal(match.fun(f)(c(10L, NA), g = rep(1L, 2), na.rm = FALSE, use.g.names = FALSE), NA_integer_)
+      # expect_equal(match.fun(f)(c(NA, 10L), g = rep(1L, 2), na.rm = FALSE, use.g.names = FALSE), NA_integer_)
+      if(f %!in% c("fmode", "ffirst")) expect_equal(match.fun(f)(c(10, NA), g = rep(1L, 2), na.rm = FALSE, use.g.names = FALSE), NA_real_)
+      if(f != "flast") expect_equal(match.fun(f)(c(NA, 10), g = rep(1L, 2), na.rm = FALSE, use.g.names = FALSE), NA_real_)
+    }
+  skip_if_not(Sys.getenv("OMP") == "TRUE")
+  for (f in c("fsum", "fmean", "fmode", "fnth", "fmedian")) {
+    expect_equal(match.fun(f)(c(10L, rep(NA_integer_, 1e5)), nthreads = 2L), 10L)
+    expect_equal(match.fun(f)(c(rep(NA_integer_, 1e5), 10L), nthreads = 2L), 10L)
+    expect_equal(match.fun(f)(c(10, rep(NA_real_, 1e5)), nthreads = 2L), 10)
+    expect_equal(match.fun(f)(c(rep(NA_real_, 1e5), 10), nthreads = 2L), 10)
+    expect_equal(match.fun(f)(c(10L, rep(NA_integer_, 1e5)), g = rep(1L, 1e5+1), nthreads = 2L, use.g.names = FALSE), 10L)
+    expect_equal(match.fun(f)(c(rep(NA_integer_, 1e5), 10L), g = rep(1L, 1e5+1), nthreads = 2L, use.g.names = FALSE), 10L)
+    expect_equal(match.fun(f)(c(10, rep(NA_real_, 1e5)), g = rep(1L, 1e5+1), nthreads = 2L, use.g.names = FALSE), 10)
+    expect_equal(match.fun(f)(c(rep(NA_real_, 1e5), 10), g = rep(1L, 1e5+1), nthreads = 2L, use.g.names = FALSE), 10)
+    # na.rm = FALSE
+    expect_equal(match.fun(f)(c(10L, rep(NA_integer_, 1e5)), na.rm = FALSE, nthreads = 2L), NA_integer_)
+    expect_equal(match.fun(f)(c(rep(NA_integer_, 1e5), 10L), na.rm = FALSE, nthreads = 2L), NA_integer_)
+    expect_equal(match.fun(f)(c(10, rep(NA_real_, 1e5)), na.rm = FALSE, nthreads = 2L), NA_real_)
+    expect_equal(match.fun(f)(c(rep(NA_real_, 1e5), 10), na.rm = FALSE, nthreads = 2L), NA_real_)
+    # Some functions are optimized and don't check here
+    # expect_equal(match.fun(f)(c(10L, rep(NA_integer_, 1e5)), g = rep(1L, 1e5+1), na.rm = FALSE, nthreads = 2L, use.g.names = FALSE), NA_integer_)
+    # expect_equal(match.fun(f)(c(rep(NA_integer_, 1e5), 10L), g = rep(1L, 1e5+1), na.rm = FALSE, nthreads = 2L, use.g.names = FALSE), NA_integer_)
+    expect_equal(match.fun(f)(c(10, rep(NA_real_, 1e5)), g = rep(1L, 1e5+1), na.rm = FALSE, nthreads = 2L, use.g.names = FALSE), NA_real_)
+    expect_equal(match.fun(f)(c(rep(NA_real_, 1e5), 10), g = rep(1L, 1e5+1), na.rm = FALSE, nthreads = 2L, use.g.names = FALSE), NA_real_)
+  }
+})
+
+test_that("fmedian ties handled properly with weights", {
+  x <- c(1, 2, 3, 4)
+  w <- c(2.5, 2.4, 3.8, 1.1)
+  expect_equal(c(fmedian(x, w = w, ties = "mean"), fmedian(x, w = w, ties = "min"), fmedian(x, w = w, ties = "max")),
+               c(2.5, 2, 3))
+  w <- c(2.5, 2.4, 3.7, 1.2)
+  expect_equal(c(fmedian(x, w = w, ties = "mean"), fmedian(x, w = w, ties = "min"), fmedian(x, w = w, ties = "max")),
+               c(2.5, 2, 3))
+})
+
+test_that("Misc bugs", {
+  expect_visible(qF(c(4L, 1L, NA), sort = FALSE))
+  expect_equal(fmatch(factor(NA, exclude = NULL), NA), 1L) # #675
+  expect_equal(fmatch(factor(NA), NA), 1L)
+  expect_visible(qsu(mtcars$mpg, mtcars$cyl, mtcars$vs, mtcars$wt))
+  expect_visible(qsu(mtcars$mpg, mtcars$cyl, mtcars$vs, mtcars$wt, higher = TRUE))
+  expect_visible(qsu(mtcars$mpg, mtcars$cyl, mtcars$vs, mtcars$wt, array = FALSE))
+  expect_visible(qsu(mtcars$mpg, mtcars$cyl, mtcars$vs, mtcars$wt, higher = TRUE, array = FALSE))
+  df1 <- data.frame(x = 1:3, y = 4:6)
+  df2 <- data.frame(x = 2:5, z = 2:5)
+  expect_visible(join(df1, df2, require = list(x = 1, y = 1, fail = "message"), verbose = 0, multiple = TRUE))
+})
+
+
+test_that("Pivot with integers", { #803
+
+  iris_long <- pivot(iris, "Species")
+  iris_long$value <- round(iris_long$value)  # Double
+  iris_long$value_int = as.integer(iris_long$value)  # Integer
+
+  for (f in c("sum", "mean")) {
+  expect_equal(
+    pivot(
+      data = iris_long,
+      ids = "Species",
+      values = "value",
+      how = "wider",  # Pivoting to wide format
+      FUN = "sum"
+    ),
+    pivot(
+      data = iris_long,
+      ids = "Species",
+      values = "value_int",
+      how = "wider",  # Pivoting to wide format
+      FUN = "sum"
+    ))
+  }
+})
+
 options(warn = 1)
